@@ -4,13 +4,15 @@
 //! It manages the project tree, command history, tabs, and various UI modes.
 
 use crate::components::{
-    palette::CommandPaletteState, settings::SettingsModalState, text_input::TextInputState,
-    updater::UpdateWizardState,
+    filter::FilterState, palette::CommandPaletteState, progress::ProgressState,
+    settings::SettingsModalState, text_input::TextInputState, updater::UpdateWizardState,
 };
+use crate::config::Config;
 use crate::events::{Action, Mode};
 use crate::project::Project;
 use crate::runner::UpdateQueue;
 use crate::settings::AppSettings;
+use crate::tree::{FlattenedTree, TreeNode, TreeSelectionState};
 use ratatui::widgets::ListState;
 use std::collections::HashSet;
 
@@ -18,7 +20,7 @@ use std::collections::HashSet;
 ///
 /// This struct maintains all mutable state including the project list, UI selections,
 /// command history, and various input states for different UI modes.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct AppState {
     /// Whether the application should quit
     pub should_quit: bool,
@@ -28,7 +30,13 @@ pub struct AppState {
     pub is_checking_updates: bool,
     /// Current application mode
     pub mode: Mode,
-    /// State of the project tree view
+    /// Root node of the hierarchical project tree
+    pub tree_root: Option<TreeNode>,
+    /// Flattened view of the tree for rendering and navigation
+    pub flattened_tree: FlattenedTree,
+    /// Tree navigation and selection state
+    pub tree_selection: TreeSelectionState,
+    /// State of the project tree view (legacy, kept for compatibility)
     pub tree_state: ListState,
     /// Filtered list of projects currently displayed
     pub projects: Vec<Project>,
@@ -56,6 +64,12 @@ pub struct AppState {
     pub settings: AppSettings,
     /// Modal state for editing settings
     pub settings_modal: SettingsModalState,
+    /// Filter/search state
+    pub filter: FilterState,
+    /// Application configuration (themes, layout, keybindings, etc.)
+    pub config: Config,
+    /// Progress tracking for command execution
+    pub progress: Option<ProgressState>,
 }
 
 /// Represents a tab pane for displaying command output
@@ -69,33 +83,8 @@ pub struct Tab {
     pub is_finished: bool,
 }
 
-impl Clone for AppState {
-    fn clone(&self) -> Self {
-        Self {
-            should_quit: self.should_quit,
-            is_scanning: self.is_scanning,
-            is_checking_updates: self.is_checking_updates,
-            mode: self.mode.clone(),
-            tree_state: self.tree_state.clone(),
-            projects: self.projects.clone(),
-            all_projects: self.all_projects.clone(),
-            collapsed_workspaces: self.collapsed_workspaces.clone(),
-            selected_projects: self.selected_projects.clone(),
-            tabs: self.tabs.clone(),
-            active_tab: self.active_tab,
-            command_history: self.command_history.clone(),
-            palette: self.palette.clone(),
-            updater: self.updater.clone(),
-            text_input: self.text_input.clone(),
-            update_queue: self.update_queue.clone(),
-            settings: self.settings.clone(),
-            settings_modal: self.settings_modal.clone(),
-        }
-    }
-}
-
-impl AppState {
-    pub fn new() -> Self {
+impl Default for AppState {
+    fn default() -> Self {
         let command_history = vec![
             "test".into(),
             "check".into(),
@@ -121,6 +110,9 @@ impl AppState {
             is_scanning: true,
             is_checking_updates: false,
             mode: Mode::Loading,
+            tree_root: None,
+            flattened_tree: FlattenedTree::new(),
+            tree_selection: TreeSelectionState::new(),
             tree_state,
             projects: Vec::new(),
             all_projects: Vec::new(),
@@ -135,6 +127,34 @@ impl AppState {
             update_queue: UpdateQueue::new(),
             settings: AppSettings::load(),
             settings_modal: SettingsModalState::new(),
+            filter: FilterState::new(),
+            config: Config::load(),
+            progress: None,
+        }
+    }
+}
+
+impl AppState {
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    /// Get the current active color scheme based on config
+    pub fn current_colors(&self) -> crate::ui::styles::Colors {
+        self.config.theme().current_scheme().colors()
+    }
+
+    /// Get keybindings from config
+    pub fn get_keybindings(&self) -> std::collections::HashMap<String, Vec<String>> {
+        self.config.keybindings.get_bindings()
+    }
+
+    /// Check if a key matches an action
+    pub fn key_matches_action(&self, key: &str, action: &str) -> bool {
+        if let Some(keys) = self.config.keybindings.get_action_keys(action) {
+            keys.iter().any(|k| k.eq_ignore_ascii_case(key))
+        } else {
+            false
         }
     }
 
@@ -150,30 +170,24 @@ impl AppState {
 
     /// Get list of projects that should be visible (excluding collapsed workspace members)
     pub fn get_visible_projects(&self) -> Vec<&Project> {
-        let mut visible = Vec::new();
-        let mut last_workspace: Option<String> = None;
+        let mut last_workspace: Option<&String> = None;
 
-        for project in &self.projects {
-            match &project.workspace_name {
+        self.projects
+            .iter()
+            .filter(|project| match &project.workspace_name {
                 Some(ws_name) => {
-                    // If workspace changed or this is the first in a workspace, always show
-                    if last_workspace.as_ref() != Some(ws_name) {
-                        visible.push(project);
-                        last_workspace = Some(ws_name.clone());
-                    } else if !self.collapsed_workspaces.contains(ws_name) {
-                        // Show member if workspace is expanded
-                        visible.push(project);
+                    let is_new_workspace = last_workspace != Some(ws_name);
+                    if is_new_workspace {
+                        last_workspace = Some(ws_name);
                     }
+                    is_new_workspace || !self.collapsed_workspaces.contains(ws_name)
                 }
                 None => {
-                    // Standalone project, always show
-                    visible.push(project);
                     last_workspace = None;
+                    true
                 }
-            }
-        }
-
-        visible
+            })
+            .collect()
     }
 
     /// Get the currently selected workspace name (if cursor is on a workspace item)
@@ -199,7 +213,9 @@ pub fn reducer(state: &mut AppState, action: Action) {
         Action::ShowHelp => handle_show_help(state),
         Action::ShowSettings => handle_show_settings(state),
         Action::CloseSettings => handle_close_settings(state),
-        Action::FinishProjectScan(projects) => handle_finish_project_scan(state, projects),
+        Action::FinishProjectScan(projects, target_dir) => {
+            handle_finish_project_scan(state, projects, target_dir)
+        }
         Action::UpdateTextInput(s) => handle_update_text_input(state, s),
         Action::SelectNext => handle_select_next(state),
         Action::SelectPrevious => handle_select_previous(state),
@@ -253,6 +269,19 @@ pub fn reducer(state: &mut AppState, action: Action) {
         Action::UpdateProjectCheckStatus(project_name, check_status) => {
             handle_update_project_check_status(state, project_name, check_status)
         }
+        Action::EnterFilterMode => handle_enter_filter_mode(state),
+        Action::ExitFilterMode => handle_exit_filter_mode(state),
+        Action::UpdateFilterInput(input) => handle_update_filter_input(state, input),
+        Action::ClearFilter => handle_clear_filter(state),
+        Action::CycleTheme => handle_cycle_theme(state),
+        Action::SetTheme(theme_name) => handle_set_theme(state, theme_name),
+        Action::IncreaseLeftPane => handle_increase_left_pane(state),
+        Action::DecreaseLeftPane => handle_decrease_left_pane(state),
+        Action::IncreaseTopRight => handle_increase_top_right(state),
+        Action::DecreaseTopRight => handle_decrease_top_right(state),
+        Action::ResetLayout => handle_reset_layout(state),
+        Action::SaveConfig => handle_save_config(state),
+        Action::ToggleShowAllFolders => handle_toggle_show_all_folders(state),
     }
 }
 
@@ -322,7 +351,10 @@ mod tests {
         project.dependencies = vec![]; // Empty dependencies - should be filtered out
         let projects = vec![project];
 
-        reducer(&mut state, Action::FinishProjectScan(projects));
+        reducer(
+            &mut state,
+            Action::FinishProjectScan(projects, ".".to_string()),
+        );
         assert!(!state.is_scanning);
         assert_eq!(state.mode, Mode::Normal);
         // Projects with empty dependencies should be filtered
@@ -333,7 +365,13 @@ mod tests {
     fn test_reducer_toggle_selection() {
         let mut state = AppState::new();
         let project = create_test_project("test1");
-        state.projects = vec![project];
+        state.projects = vec![project.clone()];
+
+        // Create a simple tree with one project node
+        let project_node = crate::tree::TreeNode::project(project, 0);
+        let mut flattened = crate::tree::FlattenedTree::new();
+        flattened.items.push((project_node, 0));
+        state.flattened_tree = flattened;
         state.tree_state.select(Some(0));
 
         // Select the project
