@@ -12,7 +12,7 @@ use carwash::ui::ui;
 
 use clap::Parser;
 use crossterm::{
-    event::{DisableMouseCapture, EnableMouseCapture, Event, KeyCode, KeyModifiers},
+    event::{Event, KeyCode, KeyModifiers},
     execute,
     terminal::{EnterAlternateScreen, LeaveAlternateScreen, disable_raw_mode, enable_raw_mode},
 };
@@ -24,13 +24,14 @@ use ratatui::{
 use std::io;
 use tokio::sync::mpsc;
 
+use anyhow::Context;
+
 #[tokio::main]
-async fn main() -> io::Result<()> {
+async fn main() -> anyhow::Result<()> {
     // Set up panic handler to ensure clean terminal restoration
     let original_hook = std::panic::take_hook();
     std::panic::set_hook(Box::new(move |panic_info| {
-        let _ = disable_raw_mode();
-        let _ = execute!(io::stdout(), LeaveAlternateScreen, DisableMouseCapture);
+        let _ = restore_terminal();
         original_hook(panic_info);
     }));
 
@@ -38,53 +39,19 @@ async fn main() -> io::Result<()> {
 
     // Check if we have a TTY (after argument parsing so --help works)
     if !crossterm::tty::IsTty::is_tty(&io::stdin()) {
-        eprintln!("Error: CarWash requires an interactive terminal (TTY).");
-        eprintln!("Please run directly in a terminal, not through pipes or redirects.");
-        std::process::exit(1);
+        anyhow::bail!("CarWash requires an interactive terminal (TTY).");
     }
 
-    let target_directory = args.target_directory.clone();
-
-    // Initialize terminal
-    if let Err(e) = enable_raw_mode() {
-        eprintln!("Error: Failed to enable raw mode: {}", e);
-        eprintln!("Make sure you're running in a proper terminal.");
-        std::process::exit(1);
-    }
-
-    let mut stdout = io::stdout();
-    if let Err(e) = execute!(stdout, EnterAlternateScreen, EnableMouseCapture) {
-        let _ = disable_raw_mode();
-        eprintln!("Error: Failed to initialize terminal: {}", e);
-        std::process::exit(1);
-    }
-
-    let backend = CrosstermBackend::new(stdout);
-    let mut terminal = match Terminal::new(backend) {
-        Ok(t) => t,
-        Err(e) => {
-            let _ = disable_raw_mode();
-            eprintln!("Error: Failed to create terminal: {}", e);
-            std::process::exit(1);
-        }
-    };
+    let mut terminal = setup_terminal().context("Failed to set up terminal")?;
 
     // Clear screen immediately to prevent any error messages from showing
     let _ = terminal.clear();
 
     let mut state = AppState::new();
-    let res = run_app(&mut terminal, &mut state, target_directory).await;
+    let res = run_app(&mut terminal, &mut state, args.target_directory).await;
 
-    // Clean up terminal
-    disable_raw_mode()?;
-    execute!(
-        terminal.backend_mut(),
-        LeaveAlternateScreen,
-        DisableMouseCapture
-    )?;
-    terminal.show_cursor()?;
+    restore_terminal().context("Failed to restore terminal")?;
 
-    // Report errors only after terminal is restored
     if let Err(err) = res {
         eprintln!("Error: {:?}", err);
         std::process::exit(1);
@@ -93,6 +60,20 @@ async fn main() -> io::Result<()> {
     Ok(())
 }
 
+fn setup_terminal() -> anyhow::Result<Terminal<CrosstermBackend<io::Stdout>>> {
+    enable_raw_mode().context("Failed to enable raw mode")?;
+    let mut stdout = io::stdout();
+    execute!(stdout, EnterAlternateScreen).context("Failed to enter alternate screen")?;
+    let backend = CrosstermBackend::new(stdout);
+    Terminal::new(backend).context("Failed to create terminal")
+}
+
+fn restore_terminal() -> anyhow::Result<()> {
+    disable_raw_mode().context("Failed to disable raw mode")?;
+    execute!(io::stdout(), LeaveAlternateScreen).context("Failed to leave alternate screen")?;
+    Ok(())
+}
+
 /// Save current dependency check progress to persistent cache
 fn save_cache_progress(state: &AppState) {
     use std::collections::HashMap;
@@ -183,10 +164,23 @@ async fn handle_event(
                     KeyCode::Char('q') => Some(Action::Quit),
                     KeyCode::Char('?') => Some(Action::ShowHelp),
                     KeyCode::Char('s') | KeyCode::Char('S') => Some(Action::ShowSettings),
+                    KeyCode::Char('t') | KeyCode::Char('T') => Some(Action::CycleTheme),
+                    KeyCode::Char('a') | KeyCode::Char('A') => Some(Action::ToggleShowAllFolders),
                     KeyCode::Char(':') => Some(Action::ShowCommandPalette),
-                    KeyCode::Char('u') => {
-                        // Open update wizard for selected project
-                        Some(Action::StartUpdateWizard)
+                    KeyCode::Char('/') => Some(Action::EnterFilterMode),
+                    KeyCode::Char('u') => Some(Action::StartUpdateWizard),
+                    // Layout adjustment controls
+                    KeyCode::Char('{') | KeyCode::Char('[') => Some(Action::DecreaseLeftPane),
+                    KeyCode::Char('}') | KeyCode::Char(']') => Some(Action::IncreaseLeftPane),
+                    KeyCode::Char('(') | KeyCode::Char('-') => Some(Action::DecreaseTopRight),
+                    KeyCode::Char(')') | KeyCode::Char('+') => Some(Action::IncreaseTopRight),
+                    KeyCode::Char('r') | KeyCode::Char('R') => {
+                        if key.modifiers.contains(KeyModifiers::SHIFT) {
+                            Some(Action::ResetLayout)
+                        } else {
+                            let mut project_list = ProjectList::new();
+                            project_list.handle_key_events(key.code, state)
+                        }
                     }
                     _ => {
                         let mut project_list = ProjectList::new();
@@ -214,6 +208,30 @@ async fn handle_event(
                 let mut settings = SettingsModal::new();
                 settings.handle_key_events(key.code, state)
             }
+            Mode::Filter => {
+                // Handle filter mode keys
+                match key.code {
+                    KeyCode::Esc => Some(Action::ExitFilterMode),
+                    KeyCode::Enter => Some(Action::ExitFilterMode),
+                    KeyCode::Char(c) => Some(Action::UpdateFilterInput(
+                        state.filter.input.clone() + &c.to_string(),
+                    )),
+                    KeyCode::Backspace => {
+                        let mut input = state.filter.input.clone();
+                        input.pop();
+                        Some(Action::UpdateFilterInput(input))
+                    }
+                    KeyCode::Up => {
+                        state.filter.select_previous();
+                        None
+                    }
+                    KeyCode::Down => {
+                        state.filter.select_next();
+                        None
+                    }
+                    _ => None,
+                }
+            }
         };
 
         if let Some(action) = action {
@@ -243,6 +261,19 @@ async fn handle_event(
     Ok(())
 }
 
+fn reset_checking_status(state: &mut AppState) {
+    for project in &mut state.all_projects {
+        if project.check_status == ProjectCheckStatus::Checking {
+            project.check_status = ProjectCheckStatus::Unchecked;
+        }
+    }
+    for project in &mut state.projects {
+        if project.check_status == ProjectCheckStatus::Checking {
+            project.check_status = ProjectCheckStatus::Unchecked;
+        }
+    }
+}
+
 async fn run_app<B: Backend>(
     terminal: &mut Terminal<B>,
     state: &mut AppState,
@@ -251,15 +282,22 @@ async fn run_app<B: Backend>(
     let (action_tx, mut action_rx) = mpsc::channel(100);
     let mut event_stream = crossterm::event::EventStream::new();
 
+    // Set up frame rate for consistent redraws (following ratatui async pattern)
+    const FRAMES_PER_SECOND: f32 = 30.0;
+    let period = std::time::Duration::from_secs_f32(1.0 / FRAMES_PER_SECOND);
+    let mut interval = tokio::time::interval(period);
+
     // Track last cache save time for periodic persistence
     let mut last_cache_save = std::time::Instant::now();
 
     let action_tx_clone = action_tx.clone();
+    let target_directory_clone = target_directory.clone();
     tokio::spawn(async move {
         // Use tokio::spawn_blocking with timeout
+        let target_dir_for_scan = target_directory_clone.clone();
         let scan_result = tokio::time::timeout(
             std::time::Duration::from_secs(30),
-            tokio::task::spawn_blocking(move || find_rust_projects(&target_directory)),
+            tokio::task::spawn_blocking(move || find_rust_projects(&target_dir_for_scan)),
         )
         .await;
 
@@ -267,33 +305,34 @@ async fn run_app<B: Backend>(
             Ok(Ok(projects)) => {
                 // Just send the scan result - queuing happens AFTER cache is loaded
                 let _ = action_tx_clone
-                    .send(Action::FinishProjectScan(projects))
+                    .send(Action::FinishProjectScan(projects, target_directory_clone))
                     .await;
             }
             Ok(Err(_)) | Err(_) => {
                 // Timeout or panic - send empty list and continue
                 let _ = action_tx_clone
-                    .send(Action::FinishProjectScan(Vec::new()))
+                    .send(Action::FinishProjectScan(Vec::new(), ".".to_string()))
                     .await;
             }
         }
     });
 
     loop {
-        // Draw UI (with timeout protection)
-        if let Err(e) = terminal.draw(|f| ui(f, state)) {
-            // If drawing fails, try to recover
-            eprintln!("Draw error: {}", e);
-            return Err(e);
-        }
-
         tokio::select! {
             // Prioritize keyboard events with biased selection
             biased;
 
+            // Handle keyboard events (events come in as they happen)
             Some(Ok(event)) = event_stream.next() => {
                 handle_event(event, state, &action_tx).await?;
             }
+            // Redraw at consistent frame rate (30 FPS)
+            _ = interval.tick() => {
+                if let Err(e) = terminal.draw(|f| ui(f, state)) {
+                    eprintln!("Draw error: {}", e);
+                    return Err(e);
+                }
+            }
             Some(action) = action_rx.recv() => {
                 match &action {
                     Action::ExecuteCommand(command) => {
@@ -305,7 +344,7 @@ async fn run_app<B: Backend>(
                             reducer(state, Action::EnterNormalMode);
                         }
                     }
-                    Action::FinishProjectScan(_) => {
+                    Action::FinishProjectScan(_, _) => {
                         // Process the scan result FIRST (copies projects to state)
                         reducer(state, action);
 
@@ -316,17 +355,7 @@ async fn run_app<B: Backend>(
                         load_cache_progress(&mut state.projects);
 
                         // Reset any "Checking" status to "Unchecked" (app was interrupted)
-                        // These projects will be queued below to resume checking
-                        for project in &mut state.all_projects {
-                            if project.check_status == ProjectCheckStatus::Checking {
-                                project.check_status = ProjectCheckStatus::Unchecked;
-                            }
-                        }
-                        for project in &mut state.projects {
-                            if project.check_status == ProjectCheckStatus::Checking {
-                                project.check_status = ProjectCheckStatus::Unchecked;
-                            }
-                        }
+                        reset_checking_status(state);
 
                         if state.settings.background_updates_enabled {
                             // NOW queue projects for background checks (after cache is loaded)
